#!/usr/bin/env python

import os 
from ctypes import *
from shutil import copyfile

is_agree=False

class KylinHaInfo(Structure):
    _fields_ = [("version", c_char * 1024), ("serial", c_char * 1024),
                ("activate_status", c_int), ("remain_trial", c_int), 
                ("activate_way", c_int), ("customer", c_char * 1024),
                ("added_trial", c_int)]



libpath = "/usr/lib/libkylin_ha_activation.so"
if os.path.exists(libpath):
    activation_lib = CDLL(libpath)
    activation_stru_init = activation_lib.ha_activation_info_init
    activation_stru_init.restype = POINTER(KylinHaInfo)

def get_ha_activation_status():
    global is_agree
    activation_info = activation_stru_init()
    activation_lib.get_ha_activate_info(activation_info)
    ha_status = activation_info.contents.activate_status
    ha_added_trial = activation_info.contents.added_trial

    status = ""
    if ha_status == 1:
        status = "in_trial"
    elif ha_status == 2:
        status = "out_trial_not_activated"
    elif ha_status == 3:
        status = "is_activated"
    else:
        status = ""

    data = {
            "ha_activate_status":status,
            "added_trial":ha_added_trial,
            "is_agree":is_agree
            }
    
    return {'action':True,'data':data}

def get_ha_activation_info():
    activation_info = activation_stru_init()
    activation_lib.get_ha_activate_info(activation_info)

    ha_status = activation_info.contents.activate_status
    status = ""
    if ha_status == 1:
        status = "in_trial"
    elif ha_status == 2:
        status = "out_trial_not_activated"
    elif ha_status == 3:
        status = "is_activated"
    else:
        status = ""

    ha_remain_trial = activation_info.contents.remain_trial

    ha_product_id = activation_info.contents.serial
    product_id = ha_product_id.decode("utf-8")

    ha_activate_way = activation_info.contents.activate_way

    ha_customer = activation_info.contents.customer
    customer = ha_customer.decode("utf-8")

    ha_added_trial = activation_info.contents.added_trial

    data = {
            "ha_activate_status": status,
            "remain_trial": ha_remain_trial,
            "product_id": product_id,
            "activate_way": ha_activate_way,
            "customer": customer,
            "added_trial": ha_added_trial
            }

    return {"action":True,"data":data}

def get_net_status_qr_path():
    activation_lib.kylin_activation_get_qrcode.restype = c_char_p
    result = c_int(0)
    qr_png = c_int(1)
    url = activation_lib.kylin_activation_get_qrcode(byref(result))
    activation_lib.create_QR(url, qr_png)
    copyfile("/tmp/kylin_ha_qr.png", "/usr/share/heartbeat-gui/ha-api/static/kylin_ha_qr.png")
    qr_path = "/kylin_ha_qr.png"

    ha_net_status = activation_lib.get_net_status()
    net_status = 0
    if ha_net_status == 1:
        net_status = 1
    else:
        net_status = 2

    data = {
            "net_status": net_status,
            "qr_path": qr_path
            }

    return {"action":True,"data":data}

def start_online_qr_activate():
    ha_activated_status = activation_lib.web_online_activation()
    activated_status = False
    if ha_activated_status == 1:
        activated_status = True
    else:
        activated_status = False

    data = {
            "activated_status": activated_status
            }


    return {"action":True,"data":data}

def start_offline_qr_activate(serial, code):
    c_serial = serial.encode('utf-8')
    c_code = code.encode('utf-8')

    ha_activated_status = activation_lib.web_offline_activation(c_serial, c_code)
    activated_status = False
    if ha_activated_status == 1:
        activated_status = True
    else:
        activated_status = False

    data = {
            "activated_status": activated_status
            }

    return {"action":True,"data":data}

def start_place_activate():
    ha_activated_status = activation_lib.web_place_activation()

    activated_status = False
    if ha_activated_status == 1:
        activated_status = True

    if ha_activated_status != 1:
        activation_lib.web_place_restore()

    data = {
            "activated_status": activated_status
            }

    return {"action":True,"data":data}

def is_agree_set_status( set_status ):
    global is_agree
    is_agree = set_status
    return
