import copy
import json
import xmltodict
import common_utils
import config

'''
    nodes_info格式
    {"nodeid": "1", "name": "HA1", "ring0_addr": "192.168.11.1", "ring1_addr": "192.168.11.4"}, 
    {"nodeid": "2", "name": "HA2", "ring0_addr": "192.168.11.2", "ring1_addr": "192.168.11.5"}

'''


def get_cluster_info():
    status, currentNode = common_utils.run_cmd("cat /etc/hostname")
    currentNode = currentNode.replace("\n", "")
    if config.is_cluster_exist():
        node_list = config.get_node_list()
        nodes = []  # 按照node存储为键值对
        index = 0
        while index < len(node_list):
            if node_list[index] == "node {":
                # 开始按照node {进行分割遍历每个node
                index += 1
                node_info = {}
                node_info_ret = {}
                while index < len(node_list) and node_list[index] != "node {":
                    d = {}
                    n = []
                    n = node_list[index].split(":")
                    d[n[0]] = n[1].strip()
                    node_info.update(d)
                    index += 1

                # 整理回传数据格式,将字典最后两项nodeid和name放到字典前两项
                count = 0
                while count < 2:
                    item = node_info.popitem()
                    node_info_ret.update({item[0]: item[1]})
                    count = count + 1
                for k, v in node_info.items():
                    if k != 'nodeid' and k != 'name':
                        if is_ipv4(v):
                            node_info_ret[k] = v
                        else:
                            node_info_ret[k] = ""
                nodes.append(node_info_ret)

        data = {'action': True, 'cluster_exist': True, 'currentNode': currentNode, 'data': nodes}
        return data
    else:
        data = {
            'action': False,
            'cluster_exist': False,
            'currentNode': currentNode,
            'error': _("Cluster not established!")
        }
        return data


def is_ipv4(ip: str) -> bool:
    """
    检查ip是否合法
    :param: ip ip地址
    :return: True 合法 False 不合法
    """
    return True if [1] * 4 == [x.isdigit() and 0 <= int(x) <= 255 for x in ip.split(".")] else False


def cluster_host_auth(username, password, data):
    node_name_str = generate_node_name_str(data)
    cmd = "echo -e \"" + username + "\\n\" \"" + password + "\" | pcs host auth " + node_name_str
    status, output = common_utils.run_cmd(cmd)
    # 返回0表示认证成功，返回0表示认证失败
    return status


def add_nodes(add_nodes_info):
    # 集群存在，执行添加节点操作
    if config.is_cluster_exist():
        # 集群存在，执行添加节点操作
        
        hb_ip_prefix = "addr="
        cmd_prefix = "pcs cluster node add "
        add_node_cmd = ""
        # 获取当前节点
        status, currentNode = common_utils.run_cmd("cat /etc/hostname")
        currentNode = currentNode.replace("\n", "")

        # 判断当前节点sbd运行状态
        cmd = "echo \"`pcs stonith sbd status`\"| grep "+currentNode + ":"
        status, stdout = common_utils.run_cmd(cmd)
        cur_node_sbd_stat=stdout.split(":")[1]
        cur_node_run_sbd = cur_node_sbd_stat.split('|')

        #若当前节点运行sbd资源
        if cur_node_run_sbd[1] == ' YES ':
            status, stdout = common_utils.run_cmd("pcs stonith sbd status --full")
            sbd_header = stdout.split('SBD header on device')
            device_info = sbd_header[1].split(":")
            sbd_device = device_info[0].strip()
            
            for node_info in add_nodes_info:
                node_str = str(node_info['name'])
                for k, v in node_info.items():
                    hb_ip_cmd = ""
                    if k != "nodeid" and k != "name":
                        hb_ip_cmd = hb_ip_cmd + ' ' + hb_ip_prefix + str(v)
                    node_str = node_str + hb_ip_cmd
                add_node_cmd = add_node_cmd + cmd_prefix + node_str + " " + "device=" + str(sbd_device) +" --start ; "
        else :
            for node_info in add_nodes_info:
                node_str = str(node_info['name'])
                for k, v in node_info.items():
                    hb_ip_cmd = ""
                    if k != "nodeid" and k != "name":
                        hb_ip_cmd = hb_ip_cmd + ' ' + hb_ip_prefix + str(v)
                    node_str = node_str + hb_ip_cmd
                add_node_cmd = add_node_cmd + cmd_prefix + node_str + " --start ; "
            
        status, output = common_utils.run_cmd(add_node_cmd)
        if status != 0:
            return {'action': False, 'error': (output)}

    else:
        # 集群不存在，执行创建集群操作
        cmd = "pcs cluster setup hacluster" + generate_node_cmd_str(add_nodes_info) + " --start"
        status, output = common_utils.run_cmd(cmd)
        if status != 0:
            if "Running cluster services" in output:
                return {'action': False, 'error': "添加的部分节点已在集群中，请先将这些节点从集群中移除，或从已在集群中节点添加本机节点。"}
            return {'action': False, 'error': "添加节点失败"}
            
    return {'action': True, 'message': _("Add node success")}


def delete_nodes(delete_nodes_info):
    remain_node_num = len(get_cluster_info()['data']) - len(delete_nodes_info)
    cmd = ""
    cluster_exist = True
    if remain_node_num > 0:
        cmd = "pcs cluster node delete " + generate_node_name_str(delete_nodes_info) + "--force"
    else:
        cluster_exist = False
        cmd = "pcs cluster destroy --all"

    status, output = common_utils.run_cmd(cmd)
    if status != 0:
        return {'action': False, 'cluster_exist': cluster_exist, 'error': (output)}
    if cluster_exist == False:
        status, output = common_utils.run_cmd("pcs host deauth")
    return {'action': True, 'cluster_exist': cluster_exist, 'message': _("Delete node success")}


def edit_hb_info(hb_info):
    cmd = "pcs cluster setup hacluster" + generate_node_cmd_str(hb_info) + " --force --start"
    status, output = common_utils.run_cmd("pcs cluster cib ra-cfg")
    edit_status, output = common_utils.run_cmd(cmd)
    status, output = common_utils.run_cmd("pcs cluster cib-push ra-cfg --config")
    status, output = common_utils.run_cmd("pcs cluster start --all")
    if edit_status == 0:
        return {"action": True, "info": _("The heartbeat configuration is modified successfully")}
    else:
        return {"action": False, "error": _("Failed to modify the heartbeat configuration")}


def get_node_run_resource_stat():
    cmd = "crm_mon -1 --as-xml"
    status, stdout = common_utils.run_cmd(cmd)
    if status != 0:
        return {'action': False, 'error': (output)}
    json_str = json.dumps(xmltodict.parse(stdout))
    json_dict = json.loads(json_str)
    nodes = json_dict['crm_mon']['nodes']
    nodes_run_resource_stat = []
    nodes_status=[]
    if "node" in nodes:
        nodes_status = nodes["node"]
    if isinstance(nodes_status,list):
        for node_status in nodes_status:
            node_run_resource_stat = get_node_run_res_stat(node_status)
            nodes_run_resource_stat.append(node_run_resource_stat)
    else:
        node_run_resource_stat = get_node_run_res_stat(nodes_status)
        nodes_run_resource_stat.append(node_run_resource_stat)

    res = {'action': True, 'data': nodes_run_resource_stat}
    return res

def get_node_run_res_stat(node_status):
    node_run_resource_stat = {}
    node_name = node_status['@name']
    node_run_resource_stat["name"] = node_name
    node_run_resource_stat["resources_running"] = node_status['@resources_running']
    return node_run_resource_stat


def generate_node_name_str(nodes_info):
    node_name = ""
    for node_info in nodes_info:
        node_name = node_name + node_info['name'] + " "
    return node_name


def generate_node_cmd_str(nodes_info):
    # 生成cmd样例参考：HA1 addr=192.168.11.1 addr=192.168.11.4 HA2 addr=192.168.11.2 addr=192.168.11.5
    hb_ip_prefix = "addr="
    cmd = ""
    for node_info in nodes_info:
        node_str = str(node_info['name'])
        for k, v in node_info.items():
            hb_ip_cmd = ""
            if k != "nodeid" and k != "name":
                hb_ip_cmd = hb_ip_cmd + ' ' + hb_ip_prefix + str(v)
            node_str = node_str + hb_ip_cmd
        cmd = cmd + ' ' + node_str
    return cmd


