import os
import settings
import sys
import addpath
import resource
import alarm
from flask import Flask
from flask import request, Response
from flask import Flask, session, redirect, url_for, request, render_template, make_response
import cluster
import json
import log
import resource
import gettext
import common_utils
import re
from flask import jsonify
import node
import scripts
import config
import sys
from datetime import timedelta
import logging
import node_management
import kylin_ha_license
import rule
import configparser
import RSA

API_VERSION = "v1"
app_name = "ha-api"
gettext.bindtextdomain(app_name)
gettext.textdomain(app_name)
gettext.install(app_name, "/usr/share/locale")
gettext.translation(app_name, "/usr/share/locale",
                    languages=["zh_CN"]).install(True)
sys.path.append("/usr/share/heartbeat-gui/ha-api")


def create_app(test_config=None):
    # create and configure the app
    app = Flask(__name__,
                static_url_path='',
                template_folder='static',
                instance_relative_config=True)

    # Set the secret key to some random bytes. Keep this really secret.
    app.secret_key = b'_5#y2L"F4Q8z\n\xec]/'
    app.config['SECRET_KEY'] = os.urandom(24)
    # Debug INFO
    app.debug = True

    @app.before_request
    def login_check():
        if request.path == '/api/v1/remotescripts':
            return
        session.permanent = True
        app.config['PERMANENT_SESSION_LIFETIME'] = timedelta(minutes=10)
        user_name = session.get('user_name')
        if user_name is None:
            if not request.path.startswith('/api/v1/login'):
                if request.path.startswith('/api/'):
                    rv = make_response('Session Timeout', 403)
                    return rv

    @app.route('/api/' + API_VERSION + '/login', methods=['POST'])
    def check_auth():
        failed_login_count = 0
        if not os.path.exists("/usr/share/heartbeat-gui/ha-api/failed_login_count"):
            with open('/usr/share/heartbeat-gui/ha-api/failed_login_count', 'w', encoding="utf-8") as file_object:
                file_object.write('0')
        else:
            try:
                with open('/usr/share/heartbeat-gui/ha-api/failed_login_count', 'r', encoding="utf-8") as file_object:
                    failed_login_count = file_object.read()
                    failed_login_count = int(failed_login_count.replace("\n", ""))
            except ValueError:
                pass
        # print(failed_login_count)

        kylin_ha_license.is_agree_set_status(False)
        data = request.json
        if data is not None and 'username' in data and 'password' in data:
            username = data['username']
            if username != 'hacluster':
                return json.dumps({
                    'action':
                    False,
                    'error':
                    _('Username is not allowed to login')
                })
                
            RSA_password = data['password']
            private_key = RSA.read_private_key()
            password = RSA.decryption(RSA_password, private_key)
            ret = common_utils.check_auth(username, password)
            if ret:
                # 更新 session 信息
                session['user_name'] = username
                session['password'] = password
                with open('/usr/share/heartbeat-gui/ha-api/failed_login_count', 'w') as file_object:
                    file_object.write('0')
               
                return json.dumps({'action': ret, 'info': _('Login success')})
            else:
                failed_login_count = failed_login_count + 1
                with open('/usr/share/heartbeat-gui/ha-api/failed_login_count', 'w') as file_object:
                    file_object.write(str(failed_login_count))
                return json.dumps({
                    'action': ret,
                    'error': _('Username or password error')
                })
        return json.dumps({'action': False, 'error': _('Login failed')})

    @app.route('/api/' + API_VERSION + '/logout', methods=['POST'])
    def logout():
        kylin_ha_license.is_agree_set_status( False )
        session.pop('user_name', None)
        session.pop('password', None)
        return json.dumps({'action': True, 'info': _('Login failed')})

    @app.route('/api/' + API_VERSION + '/user', methods=['POST'])
    def user_passwd():
        data = request.json
        if data is not None and 'password' in data:
            cmd = 'echo "hacluster:' + data['password'] +'"|chpasswd >/dev/null 2>&1'
            status, stdout = common_utils.run_cmd(cmd)
            if status != 0:
                return json.dumps({'action': False, 'error': _('Change password failed')})
            else:
                return json.dumps({'action': True, 'info': _('Change password successed')})

    # a simple page that says hello
    @app.route('/api/' + API_VERSION + '/haclusters/1',
               methods=['GET', 'POST', 'PUT'])
    def cluster_ops():
        if request.method == 'GET':
            #return Response(json.dumps(cluster.get_cluster_properties_info()), mimetype='application/json')
            return json.dumps(cluster.get_cluster_properties_info())
        if request.method == 'PUT':
            data = request.json
            ret = cluster.update_cluster_properties(data)
            return json.dumps(ret)
        else:
            return

    #local HA operation
    @app.route('/api/' + API_VERSION + '/haclusters/1/localnodes/<action>',
               methods=['GET', 'POST', 'PUT'])
    def loocalnodes_ops(action):
        if request.method == 'PUT':
            return json.dumps(cluster.operation_cluster_action(action))

    # Resource ops
    @app.route('/api/' + API_VERSION + '/haclusters/1/resources',
               methods=['GET', 'POST', 'PUT'])
    def resource_ops():
        if request.method == 'GET':
            return json.dumps(resource.get_resource_info())
        if request.method == 'POST':
            #添加资源 primitive group clone
            data = request.json
            ret = json.dumps(resource.create_resource(data))
            return ret
        else:
            return

    #获取所有资源类型
    @app.route('/api/' + API_VERSION + '/haclusters/1/metas',
               methods=['GET', 'POST', 'PUT'])
    def resource_meta_ops():
        if request.method == 'GET':
            return json.dumps(resource.get_all_rsc_metas())
        else:
            return

    #获取资源元属性
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/resources/meta_attributes/<category>',
               methods=['GET', 'POST', 'PUT'])
    def resource_meta_by_cate(category):
        if request.method == 'GET':
            return json.dumps(resource.get_rsc_meta_attributes(category))
        else:
            return

    #修改资源
    @app.route('/api/' + API_VERSION + '/haclusters/1/resources/<rsc_id>',
               methods=['GET', 'POST', 'PUT'])
    def resource_ops_by_id(rsc_id):
        if request.method == 'GET':
            return json.dumps(resource.get_resource_info_by_rsc_id(rsc_id))
        if request.method == 'PUT':
            data = request.json
            ret = resource.update_resource_attributes(rsc_id, data)
        return json.dumps(ret)

    #单个资源操作start、stop、delete、cleanup、migrate、unmigrate、location、order、colocation
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/resources/<rsc_id>/relations/<action>',
               methods=['GET', 'POST', 'PUT'])
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/resources/<rsc_id>/<action>',
               methods=['GET', 'POST', 'PUT'])
    def resource_act(rsc_id, action):
        if request.method == 'GET':
            return json.dumps(resource.get_rsc_constraints(rsc_id, action))
        if request.method == 'PUT':  #按id修改资源
            data = request.json
            ret = resource.resource_action(rsc_id, action, data)
        return json.dumps(ret)

    # hb ops
    @app.route('/api/' + API_VERSION + '/haclusters/1/hbstatus',
               methods=['GET', 'POST', 'PUT'])
    def hb_ops():
        if request.method == 'GET':
            return json.dumps({"action": True, "data": 0})
        if request.method == 'PUT':
            return
        else:
            return

    # config ops
    @app.route('/api/' + API_VERSION + '/haclusters/1/configs',
               methods=['GET', 'POST', 'PUT'])
    def config_ops():
        if request.method == 'GET':
            return json.dumps(config.get_hb_conf())
        if request.method == 'POST':
            data = request.json
            return json.dumps(config.edit_hb_info(data))
            data = {"action": True, "info": "update success"}
            return json.dumps(data)

    # logs ops
    @app.route('/api/' + API_VERSION + '/haclusters/1/logs',
               methods=['GET', 'POST', 'PUT'])
    def log_ops():
        if request.method == 'GET':
            return json.dumps(log.generate_log())
        if request.method == 'PUT':
            return
        else:
            return

    @app.route('/api/' + API_VERSION + '/haclusters/1/commands',
               methods=['GET'])
    def get_commands_list():
        command_table = {
            1: "crm_mon -1 -o",
            2: "crm_simulate -Ls",
            3: "pcs config show",
            4: "corosync-cfgtool -s"
        }
        return json.dumps({'action': True, 'data': command_table})

    @app.route('/api/' + API_VERSION + '/haclusters/1/commands/<cmd_type>',
               methods=['GET'])
    def get_commands_result(cmd_type):
        command_type = int(cmd_type)
        if not isinstance(command_type, int):
            return json.dumps({
                'action': False,
                'error': _("Please check input commands type")
            })

        command_table = {
            0: "All",
            1: "crm_mon -1 -o",
            2: "crm_simulate -Ls",
            3: "pcs config show",
            4: "corosync-cfgtool -s"
        }
        if command_type != 0 and command_type in command_table:
            command_str = command_table[command_type]
            status, output = common_utils.run_cmd(command_str)
            return json.dumps({'action': True, 'data': output})
        else:
            return json.dumps({
                'action': False,
                'error': _("Please check input commands type")
            })

    @app.route('/')
    def send_index_html():
        """
        url mapping for index html file, which with @login_required decorated
        """
        # 检查是否已经登录，如果已登录，则继续；如果未登录则检测URL 地址
        user_name = session.get('user_name')
        if user_name:
            return render_template('index.html')
        return redirect('/login')

    @app.route('/resource')
    def set_resource_html():
        """
        url mapping for index html file, which with @login_required decorated
        """
        return render_template('index.html')

    @app.route('/login')
    def set_login_html():
        """
        url mapping for index html file, which with @login_required decorated
        """
        return render_template('index.html')

    @app.route('/nodeManagement')
    def set_node_html():
        """
        url mapping for index html file, which with @login_required decorated
        """
        return render_template('index.html')

    #获取node
    @app.route('/api/' + API_VERSION + '/haclusters/1/nodes/',
               methods=['GET', 'POST', 'PUT'])
    def get_nodes():
        if request.method == 'GET':
            return json.dumps(node.get_node_info())

    #根据id取单个node
    @app.route('/api/' + API_VERSION + '/haclusters/1/nodes/<nodeid>',
               methods=['GET', 'POST', 'PUT'])
    def get_nodeid(nodeid):
        if request.method == 'GET':
            return json.dumps(node.get_nodeid_info(nodeid))

    #node状态修改
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/nodes/<node_id>/<action>',
               methods=['GET', 'POST', 'PUT'])
    def nodes_action(node_id, action):
        if request.method == 'PUT':
            return json.dumps(node.get_nodes_action(node_id, action))

    #获取资源创建
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/metas/<rsc_class>/<rsc_type>',
               methods=['GET', 'POST', 'PUT'])
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/metas/<rsc_class>/<rsc_type>/<rsc_provider>/',
               methods=['GET', 'POST', 'PUT'])
    def get_rsc_meta(rsc_class, rsc_type, rsc_provider=None):
        if request.method == 'GET':
            return json.dumps(
                resource.get_rsc_metas(rsc_class, rsc_type, rsc_provider))

    @app.route('/api/' + API_VERSION +
               '/haclusters/1/resources/meta_attributes/<category>',
               methods=['GET', 'POST', 'PUT'])
    def resource_category(category):
        if request.method == 'GET':
            return json.dumps(resource.get_rsc_meta_attributes(category))

    @app.route('/api/' + API_VERSION + '/haclusters/1/get_diskhb_device',
               methods=['GET'])
    def get_disk_hb_info():
        if request.method == 'GET':
            return json.dumps({"action": True, "data": []})

    @app.route('/api/' + API_VERSION + '/haclusters/1/scripts',
               methods=['POST','GET'])
    def generate_scripts():
        if request.method == 'POST':
            data = request.json
            return json.dumps(scripts.generate_scripts(data))
        if request.method == 'GET':
            filename = request.args.get("filename","")
            return json.dumps(scripts.is_exist(filename))

    @app.route('/api/' + API_VERSION + '/remotescripts',
             methods=['POST'])
    def generate_remote_scripts():
        data = request.json
        return json.dumps(scripts.generate_remote_scripts(data))

    @app.route('/api/' + API_VERSION + '/haclusters/1/alarms',
               methods=['POST', 'GET', 'PUT'])
    def alrms_config():
        if request.method == 'GET':
            return json.dumps(alarm.alarms_get())
        if request.method == 'POST':
            data = request.json
            return json.dumps(alarm.alarms_set(data))
        if request.method == 'PUT':
            return json.dumps(alarm.alarms_test())

    @app.route('/api/' + API_VERSION + '/haclusters/1/cluster_status',
               methods=['GET'])
    def get_cluster_status():
        if request.method == "GET":
            return json.dumps(node_management.get_cluster_info())

    @app.route('/api/' + API_VERSION + '/haclusters/1/nodes/add_nodes',
               methods=['POST'])
    def add_nodes():
        if request.method == "POST":
            user_name = session.get('user_name')
            password = session.get('password')
            data = request.json
            if node_management.cluster_host_auth(user_name, password, data['data']) == 0:
                return json.dumps(node_management.add_nodes(data['data']))
            else:
                res = {'action': False, 'error': _("Auth faild!")}
                return json.dumps(res)

    @app.route('/api/' + API_VERSION + '/haclusters/1/nodes/delete_nodes',
               methods=['POST', 'GET'])
    def delete_nodes():
        if request.method == "POST":
            data = request.json
            return json.dumps(node_management.delete_nodes(data['data']))
        if request.method == "GET":
            return json.dumps(node_management.get_node_run_resource_stat())

    @app.route('/api/' + API_VERSION + '/haclusters/1/nodes/edit_hb_info',
               methods=['POST'])
    def edit_hb_info():
        if request.method == "POST":
            data = request.json
            return json.dumps(node_management.edit_hb_info(data['data']))

    @app.route('/api/' + API_VERSION + '/haclusters/1/get_ha_activation_status',
               methods=['POST', 'GET', 'PUT'])
    def get_activation_status():
        if request.method == 'GET':
            return json.dumps(kylin_ha_license.get_ha_activation_status())
        if request.method == 'POST':
            data = request.json
            status=data["is_agree"]
            if status == True:
                kylin_ha_license.is_agree_set_status( True )
            return json.dumps(kylin_ha_license.get_ha_activation_status())

    @app.route('/api/' + API_VERSION + '/haclusters/1/get_ha_activation_info',
               methods=['POST', 'GET', 'PUT'])
    def get_activation_info():
        if request.method == 'GET':
            return json.dumps(kylin_ha_license.get_ha_activation_info())
        else:
            return

    @app.route('/api/' + API_VERSION + '/haclusters/1/get_net_status_qr_path',
               methods=['POST', 'GET', 'PUT'])
    def get_net_status_qr_path():
        if request.method == 'GET':
            return json.dumps(kylin_ha_license.get_net_status_qr_path())
        else:
            return

    @app.route('/api/' + API_VERSION + '/haclusters/1/start_qr_activate',
               methods=['POST', 'GET', 'PUT'])
    def start_qr_activate():
        if request.method == 'GET':
            return json.dumps(kylin_ha_license.start_online_qr_activate())

        if request.method == 'POST':
            data = request.json
            serial = data["serial_num"]
            code = data["activation_code"]
            return json.dumps(kylin_ha_license.start_offline_qr_activate(serial, code))

    @app.route('/api/' + API_VERSION + '/haclusters/1/start_place_activate',
               methods=['POST', 'GET', 'PUT'])
    def start_place_activate():
        if request.method == 'POST':
            up_files = request.files.getlist('files[]')
            for up_file in up_files:
                new_name = '/tmp/' + up_file.filename
                up_file.save(new_name)

            return json.dumps(kylin_ha_license.start_place_activate())
        else:
            return

    @app.route('/api/' + API_VERSION + '/rules',
            methods=['POST', 'GET', 'PUT', 'DELETE'])
    def get_rule_list():
        if request.method == 'GET':
            rscname = request.args.get("rscname","")
            return json.dumps(rule.list_rule(rscname))
        if request.method == 'DELETE':
            ruleids = request.json
            return json.dumps(rule.delete_rule(ruleids))
        if request.method == 'POST':
            data = request.json
            return json.dumps(rule.add_rule(data))
        if request.method == 'PUT':
            data = request.json
            return json.dumps(rule.update_rule(data))

    return app


if __name__ == '__main__':
    app = create_app()

    # LOG 日志
    logging.basicConfig(level=logging.DEBUG,
                   format='%(asctime)s %(levelname)s %(message)s',
                   datefmt='%a, %d %b %Y %H:%M:%S',
                   filename='/var/log/ha-api.log',
                   filemode='w')
    port_ini = "/usr/share/heartbeat-gui/ha-api/port.ini"
    if(os.path.exists(port_ini)):
        config = configparser.ConfigParser()
        config.read(port_ini)
        port = config.get("port","haapi_port")
        if not isinstance(port,int):
                if not port.isdigit():
                    port = 8088
                else:
                    port = int(port)
    else:
        port = 8088
    app.run("0.0.0.0",
            port,
            ssl_context=('/usr/share/heartbeat-gui/ha-api/.cert.pem',
                         '/usr/share/heartbeat-gui/ha-api/.key.pem'))
