# -*-coding:utf-8-*-
# !/usr/bin/python
# coding: iso-8859-15

import json
import os
import re
import common_utils
import stat
import requests
import settings

def generate_scripts(data):
    status, output = common_utils.run_cmd("crm_node -l")
    if status != 0:
        return {'action': False, 'error': output}
    nodes = str(output).strip().split("\n")
    status, output = common_utils.run_cmd("hostname")
    if status != 0:
          return {'action': False, 'error': output}
    localnodename = str(output).strip()
    result = {}
    for node in nodes:
        nodename = node.split()[1]
        if nodename == localnodename:
            ret = generate_local_scripts(data)
            if 'action' not in ret or not ret['action']:
                result[nodename]= _("Generate script failed!")
            else:
                result[nodename]= _("Generate script success!")
        else:
            url = "https://"+nodename+":"+settings.haapi_port+"/api/v1/remotescripts"
            data["nodecall"] = localnodename
            datajson = json.dumps(data)
            try:
                headers = {'Content-Type': 'application/json'}
                res = requests.post(url,data=datajson,headers=headers,verify=False)
                ret = json.loads(res.text)
                if  'action' not in ret or not ret['action']:
                    result[nodename]= _("Generate script failed!")
                else:
                    result[nodename]= _("Generate script success!")
            except Exception as e:
                result[nodename] = str(e)
    return result 
    

def generate_local_scripts(data):
    filename = data['name']
    start_command = data['start']
    stop_command = data['stop']
    monitor_command = data['monitor']
    filename_path = "/usr/lib/ocf/resource.d/pacemaker/" + filename

    script_str = '#!/bin/sh\n\
#\n\
# Author:	Kylin\n\
# License:      GNU General Public License (GPL) \n\
#\n\
#	usage: $0 {start|stop|status|monitor|validate-all|meta-data}\n\
#  OCF parameters:\n\
#\n\
##########################################################################\n\
# Initialization:\n\
#OCF_ROOT="/usr/lib/ocf"\n\
: ${OCF_FUNCTIONS_DIR=${OCF_ROOT}/lib/heartbeat}\n\
. ${OCF_FUNCTIONS_DIR}/ocf-shellfuncs\n\
#########################################################################\n\
\n\
name="' + filename + '"\n\
start_path="' +  start_command + '"\n\
stop_path="' + stop_command + '"\n\
\n\
meta_data() {\n\
	cat <<END\n\
<?xml version="1.0"?>\n\
<!DOCTYPE resource-agent SYSTEM "ra-api-1.dtd">\n\
<resource-agent name="$name">\n\
<version>1.0</version>\n\
\n\
<longdesc lang="en">\n\
This script manages $name\n\
</longdesc>\n\
<shortdesc lang="en">OCF Resource Agent compliant $name.</shortdesc>\n\
\n\
<parameters>\n\
</parameters>\n\
\n\
<actions>\n\
<action name="start"   timeout="50" />\n\
<action name="stop"    timeout="60" />\n\
<action name="monitor" depth="10"  timeout="15s" interval="30s" start-delay="1s" />\n\
<action name="validate-all"  timeout="30s" />\n\
<action name="meta-data"  timeout="5s" />\n\
</actions>\n\
</resource-agent>\n\
END\n\
}\n\
\n\
anything_status() \n\
{\n\
	ret0=`ps -ef |grep -v grep |grep "' + monitor_command  + '"|wc -l`\n\
	if [ 1  -a $ret0 -gt 0 ]; then\n\
		return $OCF_SUCCESS;\n\
	elif [ 1  -a $ret0 -eq 0 ]; then\n\
		return $OCF_NOT_RUNNING;\n\
	else\n\
		return $OCF_ERR_GENERIC;\n\
	fi\n\
}\n\
\n\
anything_start()\n\
{\n\
    anything_status\n\
    ret=$?\n\
    if [ $ret -eq 0 ]; then\n\
      ocf_log info "$name agent already running"\n\
      return $OCF_SUCCESS;\n\
    else\n\
      $start_path >/dev/null 2>&1\n\
       anything_monitor\n\
       ret1=$?\n\
       if [ $ret1 -eq 0 ]; then\n\
          ocf_log info "$name agent start ok"\n\
          return $OCF_SUCCESS;\n\
       else\n\
          ocf_log info "$name agent start failed"\n\
          return $OCF_ERR_GENERIC;\n\
       fi\n\
  fi\n\
}\n\
\n\
\n\
anything_stop()\n\
{\n\
 anything_status\n\
 ret=$?\n\
 if [ $ret -ne 7 ]; then\n\
#   ocf_log info $ret\n\
   $stop_path >/dev/null 2>&1\n\
   sleep 3 \n\
   anything_monitor\n\
   ret=$?\n\
   if [ $ret -eq 7 ]; then\n\
     ocf_log info "$name agent stop ok"\n\
     return $OCF_SUCCESS;\n\
   else\n\
     ocf_log info "$name agent stop failed"\n\
     return $OCF_ERR_GENERIC;\n\
   fi\n\
 else\n\
   ocf_log info "$name agent not running"\n\
   return $OCF_SUCCESS;\n\
 fi\n\
\n\
}\n\
\n\
anything_monitor()\n\
{\n\
  anything_status\n\
  ret=$?\n\
  #ocf_log info $ret\n\
  if [ $ret -eq 0 ]; then\n\
     ocf_log info " $name is running!"\n\
     return $OCF_SUCCESS\n\
  elif [ $ret -eq 7 ]; then\n\
     ocf_log info "$name is not running" \n\
     return $OCF_NOT_RUNNING\n\
  else\n\
    ocf_log info "$name is failed"\n\
    return $OCF_ERR_GENERIC\n\
  fi\n\
}\n\
\n\
anything_validate()\n\
{\n\
 return $OCF_SUCCESS\n\
}\n\
\n\
case $1 in\n\
    start)	anything_start\n\
		;;\n\
	\n\
    stop)	anything_stop\n\
    		;;\n\
		\n\
    status)	anything_status\n\
    		;;\n\
		\n\
    monitor)	anything_monitor\n\
    		;;\n\
		\n\
    validate-all)	anything_validate\n\
    			exit  $OCF_SUCCESS\n\
    			;;\n\
			\n\
    meta-data)	meta_data\n\
                exit  $OCF_SUCCESS\n\
  		;;\n\
    *)\n\
                ocf_log err "$0 was called with unsupported arguments: $*"\n\
                exit $OCF_ERR_UNIMPLEMENTED\n\
                ;;\n\
esac\n\
exit $?\n\
'
    try:
        file_w = open(filename_path,mode='w+')
        file_w.write(script_str)
        file_w.close 
        # chmod 755 to scripts
        os.chmod(filename_path, stat.S_IRWXU|stat.S_IRGRP|stat.S_IXGRP|stat.S_IROTH|stat.S_IXOTH)
        return {'action': True, 'info': _('Generate script success!')}
    except:
        return {'action': False, 'error': _('Generate script failed!')} 


def generate_remote_scripts(data):
    status, output = common_utils.run_cmd("crm_node -l")
    if status != 0:
        return {'action': False, 'error': output}
    nodes = str(output).strip().split("\n")
    callpri = False
    for node in nodes:
        nodename = node.split()[1]
        if nodename == data["nodecall"]:
            callpri = True
            break
    if callpri:
        ret = generate_local_scripts(data)
        if 'action' not in ret or not ret['action']:
            res = {'action': False, 'error': _('Generate script failed, please checkout network and ha service')}
        else:
            return {'action': True, 'info': _('Generate script success!')}
    else:
        res = {'action': False, 'error': _(' Permission denied!')}
    return res

def is_exist(filename):
    cmd = "pcs resource agents ocf:pacemaker"
    status, output = common_utils.run_cmd(cmd)
    if status != 0:
        return {'action': False, 'error': output}
    filelist = str(output).strip().split("\n")
    if filename in filelist:
        return {'action': False, 'error':'The file is in the pacemaker director!'}
    else:
        return {'action': True, 'info':'The file is not exist!'}

